@@ -1,6 +1,16 @@
 import warnings
 from datetime import date
-from typing import Iterable, Sequence
+from typing import Any, Iterable, Optional, Sequence, TYPE_CHECKING, Union
+
+if TYPE_CHECKING:
+    # https://github.com/microsoft/pyright/issues/1358
+    from typing import TypedDict
+else:
+    try:
+        from typing import TypedDict
+    except ImportError:
+        from typing_extensions import TypedDict
+
 
 from calcbench.api_client import (
     CompanyIdentifierScheme,
@@ -24,74 +34,39 @@ except ImportError:
     pass
 
 
-def mapped_raw(
-    company_identifiers=[],
-    all_footnotes=False,
-    point_in_time=False,
-    update_date=None,
-    metrics=[],
-    all_history=False,
-    entire_universe=False,
-    start_year=None,
-    end_year=None,
-    start_period=None,
-    end_period=None,
-    period_type=None,
-    use_fiscal_period=False,
-    include_preliminary=False,
-    all_face=False,
-    include_xbrl=True,
-    accession_id=None,
-    include_trace=False,
-):
+class TraceData(TypedDict):
+    local_name: str
+    negative_weight: str
+    XBRL_fact_value: Union[str, float, int]
+    fact_id: int
+    dimensions: int
+    trace_url: str
+
+
+class StandardizedPoint(TypedDict):
     """
-    Only called by point in time
+    Replicates MappedDataPoint on the server
     """
 
-    if update_date:
-        warnings.warn(
-            "Request updates by accession_id rather than update date",
-            DeprecationWarning,
-        )
-    if accession_id and update_date:
-        raise ValueError("Specifying accession_id and update_date is redundant.")
-
-    payload = {
-        "companiesParameters": {
-            "companyIdentifiers": list(company_identifiers),
-            "entireUniverse": entire_universe,
-        },
-        "pageParameters": {
-            "pointInTime": point_in_time,
-            "allFootnotes": all_footnotes,
-            "allface": all_face,
-            "metrics": metrics,
-            "includePreliminary": include_preliminary,
-            "includeTrace": include_trace,
-        },
-    }
-    period_parameters = {
-        "allHistory": all_history,
-        "year": start_year,
-        "endYear": end_year,
-        "period": start_period,
-        "endPeriod": end_period,
-        "periodType": period_type,
-        "useFiscalPeriod": use_fiscal_period,
-        "includeXBRL": include_xbrl,
-        "accessionID": accession_id,
-    }
-    if update_date:
-        period_parameters["updateDate"] = update_date.isoformat()
-    payload["periodParameters"] = period_parameters
-    return _json_POST("mappedData", payload)
+    metric: str
+    value: Union[str, float, int]
+    calendar_year: int
+    calendar_period: int
+    fiscal_year: int
+    fiscal_period: int
+    trace_facts: Sequence[TraceData]
+    ticker: str
+    calcbench_entity_id: int
+    filing_type: str  # 10-K, 10-Q, 8-K, PRESSRELEASE, etc.
+    is_preliminary_data: bool
+    CIK: str
+    trace_url: Optional[str]
+    period: Any  # pandas period
 
 
 def normalized_raw(
     company_identifiers: CompanyIdentifiers = [],
-    metrics: Iterable[
-        str
-    ] = [],  # type str[] Full list of metrics is @ https://www.calcbench.com/home/standardizedmetrics
+    metrics: Iterable[str] = [],
     start_year: int = None,
     start_period: PeriodArgument = None,
     end_year: int = None,
@@ -107,7 +82,10 @@ def normalized_raw(
     period_type: PeriodType = None,
     include_preliminary: bool = False,
     use_fiscal_period: bool = False,
-):
+    all_face: bool = False,
+    all_footnotes: bool = False,
+    include_xbrl: bool = False,
+) -> Sequence[StandardizedPoint]:
     """
     Standardized data.
 
@@ -189,12 +167,25 @@ def normalized_raw(
             )
         start_year = end_year = year
 
-    if period_type and period_type not in ("annual", "quarterly"):
+    if period_type and period_type not in ("annual", "quarterly", "combined"):
         raise ValueError('period_type must be either "annual" or "quarterly"')
 
     if include_preliminary and not point_in_time:
         raise ValueError("include_preliminary only works for PIT")
 
+    if metrics and (all_face or all_footnotes):
+        raise ValueError(
+            "specifying metrics with all_face or all_footnotes does not make sense"
+        )
+
+    if isinstance(metrics, str):
+        raise TypeError("metrics should be a list of strings")
+
+    if all_history and any(
+        [year, period, start_period, start_year, end_period, end_year]
+    ):
+        raise ValueError("all_history with other period arguments does not make sense")
+
     try:
         start_year = int(start_year)  # type: ignore
     except (ValueError, TypeError):
@@ -217,6 +208,9 @@ def normalized_raw(
             "includeTrace": include_trace,
             "pointInTime": point_in_time,
             "includePreliminary": include_preliminary,
+            "allFootnotes": all_footnotes,
+            "allface": all_face,
+            "includeXBRL": include_xbrl,
         },
         "periodParameters": {
             "year": start_year,
@@ -267,6 +261,7 @@ def point_in_time(
     :param all_footnotes: Retrive all of the points from the footnotes to the financials
     :param include_preliminary: Include facts from non-XBRL earnings press-releases and 8-Ks.
     :param include_xbrl: Include facts from XBRL 10-K/Qs.
+    :param include_trace: Include a URL that points to the source document.
     :return: DataFrame of facts
 
     Columns:
@@ -310,9 +305,17 @@ def point_in_time(
 
 
     """
+    if update_date:
+        warnings.warn(
+            "Request updates by accession_id rather than update date",
+            DeprecationWarning,
+        )
+    if accession_id and update_date:
+        raise ValueError("Specifying accession_id and update_date is redundant.")
 
-    data = mapped_raw(
+    data = normalized_raw(
         company_identifiers=company_identifiers,
+        all_face=all_face,
         all_footnotes=all_footnotes,
         point_in_time=True,
         update_date=update_date,
@@ -326,23 +329,16 @@ def point_in_time(
         period_type=period_type,
         use_fiscal_period=use_fiscal_period,
         include_preliminary=include_preliminary,
-        all_face=all_face,
-        include_xbrl=include_xbrl,
         accession_id=accession_id,
         include_trace=include_trace,
+        include_xbrl=include_xbrl,
     )
+
     if not data:
         return pd.DataFrame()
-    if include_trace:
-        for point in data:
-            trace_facts = point.pop("trace_facts", None)
-            if trace_facts:
-                fact_id = trace_facts[0]["fact_id"]
-                point[
-                    "trace_url"
-                ] = f"https://calcbench.com/benchmark/traceValueExcelV2?nonXBRLFactIDs={fact_id}"
-    data = pd.DataFrame(data)
 
+    data = pd.DataFrame(data)
+    data = data.drop(columns=["trace_facts"], errors="ignore")  # Those are annoying
     sort_columns = ["ticker", "metric"]
 
     if "calendar_period" in data.columns:
@@ -395,6 +391,7 @@ def normalized_dataframe(
     :param accession_id: Calcbench Accession ID
     :param year: Get data for a single year, defaults to annual data.
     :param period_type: Either "annual" or "quarterly".
+    :param trace_hyperlinks: Values are URLs to the source documents
     :return: Dataframe with the periods as the index and columns indexed by metric and ticker
 
     Usage::
@@ -420,6 +417,7 @@ def normalized_dataframe(
         period_type=period_type,
         use_fiscal_period=use_fiscal_period,
         accession_id=accession_id,
+        include_trace=trace_hyperlinks,
     )
     if not data:
         warnings.warn("No data found")
@@ -438,9 +436,7 @@ def normalized_dataframe(
         try:  # This is probably not necessary, we're doing it in the dataframe. akittredge January 2017.
             value = float(d["value"])
             if trace_hyperlinks:
-                value = '=HYPERLINK("https://www.calcbench.com/benchmark/traceValueExcelV2?metric={metric}&cid={ticker}&year={fiscal_year}&period={fiscal_period}&useFiscalPeriod=True&showLinks=true", {value})'.format(
-                    **d
-                )
+                value = f'=HYPERLINK("{d["trace_facts"][0]["trace_url"]}", {value})'
             d["value"] = value
         except (ValueError, KeyError):
             pass
